/*
 * ################################################################
 *
 * ProActive Parallel Suite(TM): The Java(TM) library for
 *    Parallel, Distributed, Multi-Core Computing for
 *    Enterprise Grids & Clouds
 *
 * Copyright (C) 1997-2011 INRIA/University of
 *                 Nice-Sophia Antipolis/ActiveEon
 * Contact: proactive@ow2.org or contact@activeeon.com
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Affero General Public License
 * as published by the Free Software Foundation; version 3 of
 * the License.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * If needed, contact us to obtain a release under GPL Version 2 or 3
 * or a different license than the AGPL.
 *
 *  Initial developer(s):               The ActiveEon Team
 *                        http://www.activeeon.com/
 *  Contributor(s):
 *
 * ################################################################
 * $$ACTIVEEON_INITIAL_DEV$$
 */
package org.ow2.proactive.virtualizing.core;

import java.util.Arrays;


/**
 * This class provides guest operating system's information about network hardware/network status and
 * guest operating system's heart beat. This way, one is able to determine if a running ( powered on )
 * virtual machine is usable. It is intended to be used exclusively as information holder.
 */
public class VMGuestStatus {
    public static enum HeartBeat {
        /** The guest heart beat answers normally */
        OK,
        /** The guest heart beat is intermittent */
        INTERMITTENT,
        /** Either the guest does not answer anymore or the guest status cannot be defined
         * for other reasons ( guest doesn't meet requirements... ) */
        UNKNOWN;
    }

    private final String[] ipAddresses, macAddresses;
    private final HeartBeat heartBeat;

    /**
     * This class is intended to be used exclusively as information holder.
     * information provided by {@link VirtualMachine2} implementation.
     * @param ipAddresses The list of identified IP Addresses attributed to the virtual machine asked for guest status. If unknown or if cannot be determined, empty String[]. A null value indicates that something went wrong.
     * @param macAddresses The list of mounted NIC's MAC Addresses of the virtual machine asked for guest status. If unknown or if cannot be determined, empty String[]. A null value indicates that something went wrong.
     * @param heartBeat The heart beat of the virtual machine asked for guest status. If unknown or if cannot be determined, empty String[]. A null value indicates that something went wrong.
     */
    public VMGuestStatus(String[] ipAddresses, String[] macAddresses, HeartBeat heartBeat) {
        this.ipAddresses = ipAddresses;
        this.macAddresses = macAddresses;
        this.heartBeat = heartBeat;
    }

    /** 
     * Returns the list of known IP Addresses attributed to the guest operating system
     * @return A string array filled in with the known guest's IP addresses. A null value indicates an error.
     */
    public String[] getIPAddresses() {
        return this.ipAddresses;
    }

    /**
     * Returns the list of mounted NIC's MAC Addresses.
     * @return A string array filled in with the known guest's mounted NIC's MAC Addresses. A null value indicates an error.
     */
    public String[] getMacAddresses() {
        return this.macAddresses;
    }

    /**
     * To get the guest operating system's heart beat.
     * @return One of {@link HeartBeat} depending on the guest operating system's heart beat. A null value indicates an error.
     */
    public HeartBeat getHeartBeat() {
        return this.heartBeat;
    }

    public String toString() {
        StringBuilder sb = new StringBuilder();
        sb.append("VM Guest heart beat: ");
        sb.append(this.heartBeat);
        sb.append(" - MAC Address(es): ");
        for (String mac : this.macAddresses) {
            sb.append(mac);
            sb.append(" ");
        }
        sb.append(" - IP Address(es): ");
        for (String ip : this.ipAddresses) {
            sb.append(ip);
            sb.append(" ");
        }
        return sb.toString();
    }

    @Override
    public boolean equals(Object dest) {
        if (dest instanceof VMGuestStatus) {
            VMGuestStatus toTest = (VMGuestStatus) dest;
            boolean result = true;
            if (this.heartBeat != null) {
                result = result && this.heartBeat.equals(toTest.getHeartBeat());
            }
            if (this.ipAddresses != null && toTest.getIPAddresses() != null) {
                result = result && Arrays.equals(this.ipAddresses, toTest.getIPAddresses());
            }
            if (this.macAddresses != null && toTest.getMacAddresses() != null) {
                result = result && Arrays.equals(this.macAddresses, toTest.getMacAddresses());
            }
            return result;
        } else {
            return false;
        }
    }

    @Override
    public int hashCode() {
        return 29 * (this.heartBeat == null ? 0 : this.heartBeat.hashCode()) + 27 *
            (this.ipAddresses == null ? 0 : Arrays.hashCode(this.ipAddresses)) + 17 *
            (this.macAddresses == null ? 0 : Arrays.hashCode(this.macAddresses));
    }
}
